@@ -1,11 +1,17 @@
 import enum
 from abc import ABC
-from typing import Tuple, Callable, Optional
+from typing import Tuple, Callable, Optional, List, Type
 
 from sflkitlib.events import EventType
-from sflkitlib.events.event import BranchEvent, FunctionExitEvent, DefEvent
+from sflkitlib.events.event import (
+    BranchEvent,
+    FunctionExitEvent,
+    DefEvent,
+    Event,
+    ConditionEvent,
+)
 
-from sflkit.analysis.analysis_type import AnalysisType
+from sflkit.analysis.analysis_type import AnalysisType, EvaluationResult
 from sflkit.analysis.spectra import Spectrum
 from sflkit.analysis.suggestion import Suggestion, Location
 from sflkit.model import scope
@@ -24,6 +30,17 @@ class Predicate(Spectrum, ABC):
         self.increase_true = 0
         self.increase_false = 0
         self.true_hits = dict()
+        self.last_evaluation = EvaluationResult.UNOBSERVED
+
+    @staticmethod
+    def default_evaluation() -> EvaluationResult:
+        return EvaluationResult.UNOBSERVED
+
+    def get_last_evaluation(self, id_: int) -> EvaluationResult:
+        if id_ not in self.true_hits:
+            return self.default_evaluation()
+        else:
+            return self.last_evaluation
 
     def finalize(self, passed: list, failed: list):
         super().finalize(passed, failed)
@@ -40,19 +57,24 @@ class Predicate(Spectrum, ABC):
                 else:
                     self.false_relevant_observed()
 
-    def hit(self, id_, event, scope_: scope.Scope = None):
-        super(Predicate, self).hit(id_, event, scope_)
+    def hit(self, id_, event: Event, scope_: scope.Scope = None):
         if id_ not in self.true_hits:
             self.true_hits[id_] = 0
-        if self._evaluate_predicate(scope_):
+        if id_ not in self.hits:
+            self.hits[id_] = 0
+        if self._evaluate_predicate(event, scope_):
             self.true_hits[id_] += 1
+            self.hits[id_] += 1
+            self.last_evaluation = EvaluationResult.TRUE
+        else:
+            self.last_evaluation = EvaluationResult.FALSE
 
     def get_metric(self, metric: Callable = None):
         if metric is None:
             metric = Predicate.IncreaseTrue
         return super().get_metric(metric)
 
-    def _evaluate_predicate(self, scope_: scope.Scope):
+    def _evaluate_predicate(self, event: Event, scope_: scope.Scope):
         return False
 
     def true_relevant_observed(self):
@@ -125,12 +147,15 @@ class Branch(Predicate):
     def events():
         return [EventType.BRANCH]
 
-    def hit(self, id_, event, scope_: scope.Scope = None):
+    def hit(self, id_, event: BranchEvent, scope_: scope.Scope = None):
         if id_ not in self.true_hits:
             self.true_hits[id_] = 0
         if event.then_id == self.then_id:
             super(Predicate, self).hit(id_, event, scope_)
             self.true_hits[id_] += 1
+            self.last_evaluation = EvaluationResult.TRUE
+        else:
+            self.last_evaluation = EvaluationResult.FALSE
 
     def get_suggestion(self, metric: Callable = None, base_dir: str = ""):
         if metric == Predicate.IncreaseFalse:
@@ -142,6 +167,9 @@ class Branch(Predicate):
             self.get_metric(metric),
         )
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{'then' if self.then else 'else'}:{self.then_id}"
+
 
 class Comp(enum.Enum):
     LT = 0
@@ -165,6 +193,12 @@ class Comp(enum.Enum):
         elif self == Comp.NE:
             return x != y
 
+    def __repr__(self):
+        return self.name
+
+    def __str__(self):
+        return self.name
+
 
 class Comparison(Predicate, ABC):
     def __init__(self, file, line, op: Comp):
@@ -180,7 +214,7 @@ class Comparison(Predicate, ABC):
             EventType.FUNCTION_ERROR,
         ]
 
-    def _evaluate_predicate(self, scope_: scope.Scope) -> bool:
+    def _evaluate_predicate(self, event: Event, scope_: scope.Scope) -> bool:
         return self._compare(self._get_first(scope_), self._get_second(scope_))
 
     def _compare(self, first, second) -> bool:
@@ -194,7 +228,7 @@ class Comparison(Predicate, ABC):
 
 
 class ScalarPair(Comparison):
-    def __init__(self, event, op: Comp, var: str):
+    def __init__(self, event: DefEvent, op: Comp, var: str):
         super().__init__(event.file, event.line, op)
         self.var1 = event.var
         self.var2 = var
@@ -209,9 +243,12 @@ class ScalarPair(Comparison):
     def _get_second(self, scope_: scope.Scope):
         return scope_.value(self.var2)
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var1}{self.op}{self.var2}"
+
 
 class VariablePredicate(Comparison):
-    def __init__(self, event, op: Comp):
+    def __init__(self, event: DefEvent, op: Comp):
         super().__init__(event.file, event.line, op)
         self.var = event.var
 
@@ -228,6 +265,9 @@ class VariablePredicate(Comparison):
     def _get_first(self, scope_: scope.Scope):
         return scope_.value(self.var)
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var}{self.op}0"
+
 
 class NonePredicate(Comparison):
     def __init__(self, event: DefEvent):
@@ -250,6 +290,9 @@ class NonePredicate(Comparison):
     def _get_second(self, scope_: scope.Scope):
         return None
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var}"
+
 
 class ReturnPredicate(Comparison):
     def __init__(self, event: FunctionExitEvent, op: Comp, value: Optional[int] = 0):
@@ -275,6 +318,9 @@ class ReturnPredicate(Comparison):
     def _get_second(self, scope_: scope.Scope):
         return self.value
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.function}{self.op}{self.value}"
+
 
 class EmptyStringPredicate(Comparison):
     def __init__(self, event: DefEvent):
@@ -297,6 +343,9 @@ class EmptyStringPredicate(Comparison):
     def _get_second(self, scope_: scope.Scope):
         return ""
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var}"
+
 
 class EmptyBytesPredicate(Comparison):
     def __init__(self, event: DefEvent):
@@ -319,6 +368,9 @@ class EmptyBytesPredicate(Comparison):
     def _get_second(self, scope_: scope.Scope):
         return b""
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var}"
+
 
 class FunctionPredicate(Predicate, ABC):
     def __init__(self, file, line, var: str, predicate: callable):
@@ -332,7 +384,7 @@ class FunctionPredicate(Predicate, ABC):
             EventType.DEF,
         ]
 
-    def _evaluate_predicate(self, scope_: scope.Scope):
+    def _evaluate_predicate(self, event: Event, scope_: scope.Scope):
         value = scope_.value(self.var)
         return isinstance(value, str) and self.predicate(scope_.value(self.var))
 
@@ -345,6 +397,9 @@ class IsAsciiPredicate(FunctionPredicate):
     def analysis_type():
         return AnalysisType.ASCII_STRING
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var}"
+
 
 class ContainsDigitPredicate(FunctionPredicate):
     def __init__(self, event: DefEvent):
@@ -358,6 +413,9 @@ class ContainsDigitPredicate(FunctionPredicate):
     def analysis_type():
         return AnalysisType.DIGIT_STRING
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var}"
+
 
 class ContainsSpecialPredicate(FunctionPredicate):
     def __init__(self, event: DefEvent):
@@ -371,6 +429,9 @@ class ContainsSpecialPredicate(FunctionPredicate):
     def analysis_type():
         return AnalysisType.SPECIAL_STRING
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var}"
+
 
 class Condition(Predicate):
     def __init__(self, file: str, line: int, condition: str):
@@ -385,9 +446,46 @@ class Condition(Predicate):
     def events():
         return [EventType.CONDITION]
 
-    def hit(self, id_, event, scope_: scope.Scope = None):
+    def hit(self, id_, event: ConditionEvent, scope_: scope.Scope = None):
         super(Predicate, self).hit(id_, event, scope_)
         if id_ not in self.true_hits:
             self.true_hits[id_] = 0
         if event.value:
             self.true_hits[id_] += 1
+            self.last_evaluation = EvaluationResult.TRUE
+        else:
+            self.last_evaluation = EvaluationResult.FALSE
+
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.condition}"
+
+
+class FunctionErrorPredicate(Predicate):
+    def __init__(self, file, line, function):
+        super().__init__(file, line)
+        self.function = function
+
+    @staticmethod
+    def analysis_type() -> AnalysisType:
+        return AnalysisType.FUNCTION_ERROR
+
+    @staticmethod
+    def events() -> List[Type]:
+        return [
+            EventType.FUNCTION_ENTER,
+            EventType.FUNCTION_ERROR,
+            EventType.FUNCTION_EXIT,
+        ]
+
+    def _evaluate_predicate(self, event: Event, scope_: scope.Scope):
+        return event.event_type == EventType.FUNCTION_ERROR
+
+    def get_suggestion(self, metric: Callable = None, base_dir: str = ""):
+        finder = self.function_finder(self.file, self.line, self.function)
+        return Suggestion(
+            [Location(self.file, line) for line in finder.get_locations(base_dir)],
+            self.get_metric(metric),
+        )
+
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.function}:{self.line}"
