@@ -12,9 +12,10 @@ from sflkitlib.events.event import (
     LoopHitEvent,
     DefEvent,
     UseEvent,
+    LenEvent,
 )
 
-from sflkit.analysis.analysis_type import AnalysisObject, AnalysisType
+from sflkit.analysis.analysis_type import AnalysisObject, AnalysisType, EvaluationResult
 from sflkit.analysis.suggestion import Suggestion, Location
 from sflkit.model.scope import Scope
 
@@ -38,7 +39,20 @@ class Spectrum(AnalysisObject, ABC):
         self.failed = failed_observed + failed_not_observed
         self.failed_observed = failed_observed
         self.failed_not_observed = failed_not_observed
-        self.hits = dict()
+        self.last_evaluation: EvaluationResult = EvaluationResult.FALSE
+
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}"
+
+    @staticmethod
+    def default_evaluation() -> EvaluationResult:
+        return EvaluationResult.FALSE
+
+    def get_last_evaluation(self, id_: int) -> EvaluationResult:
+        if id_ not in self.hits:
+            return self.default_evaluation()
+        else:
+            return self.last_evaluation
 
     def get_metric(self, metric: Callable = None):
         if metric is None:
@@ -59,6 +73,7 @@ class Spectrum(AnalysisObject, ABC):
         self.suspiciousness = self.get_metric(metric)
 
     def hit(self, id_, event, scope_: Scope = None):
+        self.last_evaluation = EvaluationResult.TRUE
         if id_ not in self.hits:
             self.hits[id_] = 1
         else:
@@ -483,6 +498,9 @@ class Function(Spectrum):
             self.get_metric(metric),
         )
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.function}:{self.line}"
+
 
 class DefUse(Spectrum):
     def __init__(self, def_event: DefEvent, use_event: UseEvent):
@@ -505,8 +523,23 @@ class DefUse(Spectrum):
             self.get_metric(metric),
         )
 
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.use_file}:{self.use_line}:{self.var}"
 
-class Loop(Spectrum):
+
+class ModifiableSpectrum(Spectrum, ABC):
+    def finalize_evaluation(self, evaluate: Callable, passed: list, failed: list):
+        for event_file in failed:
+            if event_file in self.hits and any(map(evaluate, self.hits[event_file])):
+                self.fail_observed()
+        for event_file in passed:
+            if event_file in self.hits and any(map(evaluate, self.hits[event_file])):
+                self.pass_observed()
+        self.set_passed(len(passed))
+        self.set_failed(len(failed))
+
+
+class Loop(ModifiableSpectrum):
     def __init__(
         self,
         event: LoopBeginEvent | LoopHitEvent | LoopEndEvent,
@@ -550,18 +583,7 @@ class Loop(Spectrum):
             self.hits[id_].append(hits)
 
     def finalize(self, passed: list, failed: list):
-        for event_file in failed:
-            if event_file in self.hits and any(
-                map(self.evaluate_hit, self.hits[event_file])
-            ):
-                self.fail_observed()
-        for event_file in passed:
-            if event_file in self.hits and any(
-                map(self.evaluate_hit, self.hits[event_file])
-            ):
-                self.pass_observed()
-        self.set_passed(len(passed))
-        self.set_failed(len(failed))
+        self.finalize_evaluation(self.evaluate_hit, passed, failed)
 
     def get_suggestion(self, metric: Callable = None, base_dir: str = ""):
         finder = self.loop_finder(self.file, self.line)
@@ -569,3 +591,49 @@ class Loop(Spectrum):
             [Location(self.file, line) for line in finder.get_locations(base_dir)],
             self.get_metric(metric),
         )
+
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}"
+
+
+class Length(ModifiableSpectrum):
+    def __init__(
+        self,
+        event: LenEvent,
+        evaluate_hit: Optional[Callable] = None,
+    ):
+        super().__init__(event.file, event.line)
+        self.var = event.var
+        self.evaluate_length = evaluate_hit if evaluate_hit else self.evaluate_length_0
+
+    @staticmethod
+    def evaluate_length_0(x):
+        return x == 0
+
+    @staticmethod
+    def evaluate_length_1(x):
+        return x == 1
+
+    @staticmethod
+    def evaluate_length_more(x):
+        return x > 1
+
+    @staticmethod
+    def analysis_type():
+        return AnalysisType.LENGTH
+
+    @staticmethod
+    def events():
+        return [EventType.LEN]
+
+    def hit(self, id_, event, scope_: Scope = None):
+        if id_ not in self.hits:
+            self.hits[id_] = [event.length]
+        else:
+            self.hits[id_].append(event.length)
+
+    def finalize(self, passed: list, failed: list):
+        self.finalize_evaluation(self.evaluate_length, passed, failed)
+
+    def __str__(self):
+        return f"{self.analysis_type()}:{self.file}:{self.line}:{self.var}:{self.evaluate_length.__name__}"
