@@ -1,5 +1,6 @@
 import configparser
 import csv
+import hashlib
 import os.path
 import queue
 from pathlib import Path
@@ -24,6 +25,7 @@ from sflkit.language.meta import (
     MetaVisitor,
 )
 from sflkit.language.visitor import ASTVisitor
+from sflkit.mapping import EventMapping, InstrumentationError
 from sflkit.model.event_file import EventFile
 from sflkit.runners import RunnerType
 
@@ -51,6 +53,8 @@ class Config:
 
     [instrumentation]
     path=/path/to/the/instrumented/subject
+    include=file(,file)*                    : Files to include in the instrumentation (other files will automatically
+                                              be excluded), should be a python re pattern
     exclude=file(,file)*                    : Files to exclude from the instrumentation, should be a python re pattern
 
     [test]
@@ -68,9 +72,12 @@ class Config:
         self.visitor = None
         self.passing = list()
         self.failing = list()
+        self.instrument_include = list()
         self.instrument_exclude = list()
         self.instrument_working = None
         self.runner = None
+        self.mapping = None
+        self.mapping_path = None
         if path:
             if isinstance(path, configparser.ConfigParser):
                 config = path
@@ -80,9 +87,11 @@ class Config:
             try:
                 # target section
                 target = config["target"]
-                self.target_path = Path(target["path"])
-                self.language = Language[target["language"].upper()]
-                self.language.setup()
+                if "path" in target:
+                    self.target_path = Path(target["path"])
+                if "language" in target:
+                    self.language = Language[target["language"].upper()]
+                    self.language.setup()
 
                 # events section
                 events = config["events"]
@@ -105,7 +114,7 @@ class Config:
                     self.events = {
                         e for p in self.predicates for e in analysis_mapping[p].events()
                     }
-                else:
+                elif "events" in events:
                     self.factory = CombinationFactory(list())
                     # get the events
                     self.events = list(
@@ -118,6 +127,7 @@ class Config:
                 self.meta_visitor = CombinationVisitor(
                     self.language,
                     IDGenerator(),
+                    IDGenerator(),
                     TmpGenerator(),
                     [self.language.meta_visitors[e] for e in self.events],
                 )
@@ -134,22 +144,40 @@ class Config:
                     self.metrics = [Spectrum.Ochiai]
 
                 run_id_generator = IDGenerator()
+                if "mapping" in events:
+                    self.mapping_path = Path(events["mapping"])
+                try:
+                    self.mapping = EventMapping.load(self)
+                except InstrumentationError:
+                    self.mapping = EventMapping(path=self.mapping_path)
                 if "passing" in events:
                     self.passing = self.get_event_files(
                         list(csv.reader([events["passing"]]))[0],
                         run_id_generator,
+                        self.mapping,
                         False,
                     )
                 if "failing" in events:
                     self.failing = self.get_event_files(
-                        list(csv.reader([events["failing"]]))[0], run_id_generator, True
+                        list(csv.reader([events["failing"]]))[0],
+                        run_id_generator,
+                        self.mapping,
+                        True,
                     )
                 # instrumentation section
                 instrument = config["instrumentation"]
+                if "include" in instrument:
+                    self.instrument_include = list(csv.reader([instrument["include"]]))[
+                        0
+                    ]
+                    while "" in self.instrument_include:
+                        self.instrument_include.remove("")
                 if "exclude" in instrument:
                     self.instrument_exclude = list(csv.reader([instrument["exclude"]]))[
                         0
                     ]
+                    while "" in self.instrument_exclude:
+                        self.instrument_exclude.remove("")
                 self.instrument_working = Path(instrument["path"])
 
                 # test section
@@ -173,6 +201,8 @@ class Config:
         visitor: ASTVisitor = None,
         passing: List[EventFile] = None,
         failing: List[EventFile] = None,
+        mapping: EventMapping = None,
+        instrument_include: List[str] = None,
         instrument_exclude: List[str] = None,
         instrument_working: str = None,
         runner: RunnerType = None,
@@ -190,13 +220,18 @@ class Config:
         conf.visitor = visitor
         conf.passing = passing if passing else list()
         conf.failing = failing if failing else list()
+        conf.instrument_include = instrument_include if instrument_include else list()
         conf.instrument_exclude = instrument_exclude if instrument_exclude else list()
         conf.instrument_working = instrument_working
         conf.runner = runner
+        if mapping:
+            conf.mapping = mapping
+            if mapping.path:
+                conf.mapping_path = mapping.path
         return conf
 
     @staticmethod
-    def get_event_files(files, run_id_generator, failing):
+    def get_event_files(files, run_id_generator, mapping: EventMapping, failing):
         file_queue = queue.Queue()
         for f in files:
             file_queue.put(f)
@@ -210,12 +245,20 @@ class Config:
                 elif os.path.isfile(element) and not os.path.islink(element):
                     result.append(
                         EventFile(
-                            element, run_id_generator.get_next_id(), failing=failing
+                            element,
+                            run_id_generator.get_next_id(),
+                            mapping,
+                            failing=failing,
                         )
                     )
             else:
                 result.append(
-                    EventFile(element, run_id_generator.get_next_id(), failing=failing)
+                    EventFile(
+                        element,
+                        run_id_generator.get_next_id(),
+                        mapping,
+                        failing=failing,
+                    )
                 )
         return result
 
@@ -228,7 +271,9 @@ class Config:
         metrics=None,
         passing=None,
         failing=None,
+        mapping_path=None,
         working=None,
+        include=None,
         exclude=None,
         runner=None,
     ):
@@ -252,8 +297,12 @@ class Config:
             conf["events"]["passing"] = passing
         if failing:
             conf["events"]["failing"] = failing
+        if mapping_path:
+            conf["events"]["mapping"] = mapping_path
         if working:
             conf["instrumentation"]["path"] = working
+        if include:
+            conf["instrumentation"]["include"] = include
         if exclude:
             conf["instrumentation"]["exclude"] = exclude
         if runner:
@@ -282,8 +331,12 @@ class Config:
             conf["events"]["passing"] = ",".join(e.path for e in self.passing)
         if self.failing:
             conf["events"]["failing"] = ",".join(e.path for e in self.failing)
+        if self.mapping_path:
+            conf["events"]["mapping"] = str(self.mapping_path)
         if self.instrument_working:
             conf["instrumentation"]["path"] = str(self.instrument_working)
+        if self.instrument_include:
+            conf["instrumentation"]["include"] = ",".join(self.instrument_include)
         if self.instrument_exclude:
             conf["instrumentation"]["exclude"] = ",".join(self.instrument_exclude)
         if self.runner:
@@ -292,6 +345,13 @@ class Config:
         with open(path, "w") as fp:
             conf.write(fp)
 
+    def identifier(self):
+        return hash_identifier(self.target_path)
+
+
+def hash_identifier(path: os.PathLike):
+    return hashlib.md5(str(path).encode("utf-8")).hexdigest()
+
 
 def parse_config(path: str) -> Config:
     return Config(path)
