@@ -1,15 +1,16 @@
-import json
 from abc import abstractmethod
-from typing import List
+from pathlib import Path
+from typing import List, Optional
 
-from sflkitlib.events import event
+from sflkit import Config
 from sflkit.language.visitor import ASTVisitor
+from sflkit.mapping import EventMapping
 
 
 class Instrumentation:
-    def __init__(self, visitor: ASTVisitor):
+    def __init__(self, visitor: ASTVisitor, mapping_path: Optional[Path] = None):
         self.visitor = visitor
-        self.events = list()
+        self.events = EventMapping(path=mapping_path)
 
     @abstractmethod
     def instrument(
@@ -17,9 +18,8 @@ class Instrumentation:
     ):
         raise NotImplementedError()
 
-    def dump_events(self, out_file):
-        with open(out_file, "w") as fp:
-            json.dump(self.events, fp, cls=event.EventEncoder)
+    def dump_events(self, config: Config):
+        self.events.write(config)
 
 
 __all__ = ["dir_instrumentation", "file_instrumentation", "Instrumentation"]
