@@ -156,7 +156,7 @@ class PythonConditionExtract(ast.NodeVisitor, ConditionExtract):
         e = ConditionEvent(
             self.file,
             val.lineno,
-            self.factory.id_generator.get_next_id(),
+            self.factory.event_id_generator.get_next_id(),
             expression,
             tmp_var=var,
         )
@@ -173,6 +173,7 @@ class PythonConditionExtract(ast.NodeVisitor, ConditionExtract):
                         ],
                         value=val,
                         type_comment=None,
+                        lineno=0,
                     ),
                     self.factory.get_event_call(e),
                 ],
@@ -200,7 +201,7 @@ class PythonConditionExtract(ast.NodeVisitor, ConditionExtract):
         es = list()
         if isinstance(node.op, ast.And):
             _, use_, assign, e = tmp_vars[-1]
-            es.append(e)
+            es += e
             variables = [use_]
             for var_, use_, assign_, events in reversed(tmp_vars[:-1]):
                 variables.append(use_)
