@@ -15,6 +15,7 @@ from sflkitlib.events.event import (
     LoopEndEvent,
     UseEvent,
     ConditionEvent,
+    LenEvent,
 )
 
 from sflkit.language.meta import MetaVisitor, Injection, IDGenerator, TmpGenerator
@@ -44,9 +45,15 @@ def get_call(function, *args) -> Expr:
 
 class PythonEventFactory(MetaVisitor, NodeVisitor):
     def __init__(
-        self, language, id_generator: IDGenerator, tmp_generator: TmpGenerator
+        self,
+        language,
+        event_id_generator: IDGenerator,
+        funtion_id_generator: IDGenerator,
+        tmp_generator: TmpGenerator,
     ):
-        super().__init__(language, id_generator, tmp_generator)
+        super().__init__(
+            language, event_id_generator, funtion_id_generator, tmp_generator
+        )
 
     def visit_start(self, *args) -> Injection:
         return self.visit(*args)
@@ -58,7 +65,7 @@ class PythonEventFactory(MetaVisitor, NodeVisitor):
         pass
 
     def get_event_call(self, event: Event):
-        return get_call(self.get_function(), event.file, event.line, event.id_)
+        return get_call(self.get_function(), event.event_id)
 
 
 class LineEventFactory(PythonEventFactory):
@@ -66,7 +73,9 @@ class LineEventFactory(PythonEventFactory):
         return "add_line_event"
 
     def visit_line(self, node: AST) -> Injection:
-        line_event = LineEvent(self.file, node.lineno, self.id_generator.get_next_id())
+        line_event = LineEvent(
+            self.file, node.lineno, self.event_id_generator.get_next_id()
+        )
         return Injection(pre=[self.get_event_call(line_event)], events=[line_event])
 
     def visit_Assign(self, node: Assign) -> Injection:
@@ -138,20 +147,16 @@ class LineEventFactory(PythonEventFactory):
 
 class BranchEventFactory(PythonEventFactory):
     def __init__(
-        self, language, id_generator: IDGenerator, tmp_generator: TmpGenerator
+        self,
+        language,
+        event_id_generator: IDGenerator,
+        funtion_id_generator: IDGenerator,
+        tmp_generator: TmpGenerator,
     ):
-        super().__init__(language, id_generator, tmp_generator)
-        self.branch_id = 0
-
-    def get_event_call(self, event: BranchEvent):
-        return get_call(
-            self.get_function(),
-            event.file,
-            event.line,
-            event.id_,
-            event.then_id,
-            event.else_id,
+        super().__init__(
+            language, event_id_generator, funtion_id_generator, tmp_generator
         )
+        self.branch_id = 0
 
     def get_function(self):
         return "add_branch_event"
@@ -160,14 +165,14 @@ class BranchEventFactory(PythonEventFactory):
         then_branch_event = BranchEvent(
             self.file,
             node.lineno,
-            self.id_generator.get_next_id(),
+            self.event_id_generator.get_next_id(),
             self.branch_id,
             self.branch_id + 1,
         )
         else_branch_event = BranchEvent(
             self.file,
             node.lineno,
-            self.id_generator.get_next_id(),
+            self.event_id_generator.get_next_id(),
             self.branch_id + 1,
             self.branch_id,
         )
@@ -196,19 +201,31 @@ class BranchEventFactory(PythonEventFactory):
 
     def visit_ExceptHandler(self, node: ExceptHandler) -> Injection:
         branch_event = BranchEvent(
-            self.file, node.lineno, self.id_generator.get_next_id(), self.branch_id, -1
+            self.file,
+            node.lineno,
+            self.event_id_generator.get_next_id(),
+            self.branch_id,
+            -1,
         )
         return Injection(
             body=[self.get_event_call(branch_event)], events=[branch_event]
         )
 
     def visit_Try(self, node: Try) -> Injection:
-        else_branch_event = BranchEvent(
-            self.file, node.lineno, self.id_generator.get_next_id(), self.branch_id, -1
-        )
-        return Injection(
-            orelse=[self.get_event_call(else_branch_event)], events=[else_branch_event]
-        )
+        if node.handlers:
+            else_branch_event = BranchEvent(
+                self.file,
+                node.lineno,
+                self.event_id_generator.get_next_id(),
+                self.branch_id,
+                -1,
+            )
+            return Injection(
+                orelse=[self.get_event_call(else_branch_event)],
+                events=[else_branch_event],
+            )
+        else:
+            return Injection()
 
 
 class DefEventFactory(PythonEventFactory):
@@ -216,9 +233,7 @@ class DefEventFactory(PythonEventFactory):
         return "add_def_event"
 
     def get_event_call(self, event: DefEvent):
-        call = get_call(
-            self.get_function(), event.file, event.line, event.id_, event.var
-        )
+        call = get_call(self.get_function(), event.event_id)
         assert isinstance(call.value, Call)
         call.value.args.append(
             Call(
@@ -257,14 +272,7 @@ class DefEventFactory(PythonEventFactory):
         def_events = list()
         for argument in self.variable_extract.visit(node.args):
             if argument != "self":
-                def_events.append(
-                    DefEvent(
-                        self.file,
-                        node.lineno,
-                        self.id_generator.get_next_id(),
-                        argument,
-                    )
-                )
+                def_events.append(self.get_event(node, argument))
         return Injection(
             body=[self.get_event_call(e) for e in def_events], events=def_events
         )
@@ -275,14 +283,17 @@ class DefEventFactory(PythonEventFactory):
     def visit_AsyncFunctionDef(self, node: AsyncFunctionDef) -> Injection:
         return self.visit_function(node)
 
+    def get_event(self, node: typing.Union[Assign, AnnAssign, AugAssign], var: str):
+        return DefEvent(
+            self.file, node.lineno, self.event_id_generator.get_next_id(), var
+        )
+
     def visit_var_assign(
         self, node: typing.Union[Assign, AnnAssign, AugAssign], vars_: list
     ) -> Injection:
         def_events = list()
         for var in vars_:
-            def_events.append(
-                DefEvent(self.file, node.lineno, self.id_generator.get_next_id(), var)
-            )
+            def_events.append(self.get_event(node, var))
         return Injection(
             post=[self.get_event_call(e) for e in def_events], events=def_events
         )
@@ -309,9 +320,7 @@ class DefEventFactory(PythonEventFactory):
         vars_ = self.variable_extract.visit(node.target)
         def_events = list()
         for var in vars_:
-            def_events.append(
-                DefEvent(self.file, node.lineno, self.id_generator.get_next_id(), var)
-            )
+            def_events.append(self.get_event(node, var))
         return Injection(
             body=[self.get_event_call(e) for e in def_events], events=def_events
         )
@@ -327,11 +336,7 @@ class DefEventFactory(PythonEventFactory):
         for item in node.items:
             if item.optional_vars:
                 for var in self.variable_extract.visit(item.optional_vars):
-                    def_events.append(
-                        DefEvent(
-                            self.file, node.lineno, self.id_generator.get_next_id(), var
-                        )
-                    )
+                    def_events.append(self.get_event(node, var))
         if def_events:
             return Injection(
                 body=[self.get_event_call(e) for e in def_events], events=def_events
@@ -349,19 +354,24 @@ class FunctionEventFactory(PythonEventFactory):
     functions: typing.Dict[AST, int] = dict()
     functions_exit_id: typing.Dict[AST, int] = dict()
     function_var: typing.Dict[AST, str] = dict()
-    function_id: int = 0
 
     def __init__(
-        self, language, id_generator: IDGenerator, tmp_generator: TmpGenerator
+        self,
+        language,
+        event_id_generator: IDGenerator,
+        funtion_id_generator: IDGenerator,
+        tmp_generator: TmpGenerator,
     ):
-        super().__init__(language, id_generator, tmp_generator)
+        super().__init__(
+            language, event_id_generator, funtion_id_generator, tmp_generator
+        )
         self.function_stack = list()
 
-    @staticmethod
-    def get_function_id(node: AST) -> int:
+    def get_function_id(self, node: AST) -> int:
         if node not in FunctionEventFactory.functions:
-            FunctionEventFactory.functions[node] = FunctionEventFactory.function_id
-            FunctionEventFactory.function_id += 1
+            FunctionEventFactory.functions[
+                node
+            ] = self.function_id_generator.get_next_id()
         return FunctionEventFactory.functions[node]
 
     @staticmethod
@@ -387,25 +397,15 @@ class FunctionEnterEventFactory(FunctionEventFactory):
     def get_function(self):
         return "add_function_enter_event"
 
-    def get_event_call(self, event: FunctionEnterEvent):
-        return get_call(
-            self.get_function(),
-            event.file,
-            event.line,
-            event.id_,
-            event.function_id,
-            event.function,
-        )
-
     def visit_function(
         self, node: typing.Union[FunctionDef, AsyncFunctionDef]
     ) -> Injection:
         function_enter_event = FunctionEnterEvent(
             self.file,
             node.lineno,
-            self.id_generator.get_next_id(),
-            self.get_function_id(node),
+            self.event_id_generator.get_next_id(),
             node.name,
+            self.get_function_id(node),
         )
         function_var = self.get_function_var(node, self.tmp_generator)
         return Injection(
@@ -418,6 +418,7 @@ class FunctionEnterEventFactory(FunctionEventFactory):
                         ),
                     ],
                     value=ast.Constant(value=None),
+                    lineno=0,
                 ),
             ],
             events=[function_enter_event],
@@ -437,11 +438,7 @@ class FunctionExitEventFactor(FunctionEventFactory):
     def get_event_call(self, event: FunctionExitEvent):
         call = get_call(
             self.get_function(),
-            event.file,
-            event.line,
-            event.id_,
-            event.function_id,
-            event.function,
+            event.event_id,
         )
         assert isinstance(call.value, Call)
         call.value.args.append(Name(id=event.tmp_var))
@@ -469,9 +466,9 @@ class FunctionExitEventFactor(FunctionEventFactory):
         function_exit_event = FunctionExitEvent(
             self.file,
             node.lineno,
-            self.get_function_event_id(node, self.id_generator),
-            self.get_function_id(node),
+            self.get_function_event_id(node, self.event_id_generator),
             node.name,
+            self.get_function_id(node),
             tmp_var=self.get_function_var(node, self.tmp_generator),
         )
         return Injection(
@@ -491,9 +488,9 @@ class FunctionExitEventFactor(FunctionEventFactory):
         function_exit_event = FunctionExitEvent(
             self.file,
             node.lineno,
-            self.get_function_event_id(node, self.id_generator),
-            self.get_function_id(function),
+            self.get_function_event_id(node, self.event_id_generator),
             function.name,
+            self.get_function_id(function),
             tmp_var=function_var,
         )
         return Injection(
@@ -505,6 +502,7 @@ class FunctionExitEventFactor(FunctionEventFactory):
                         ),
                     ],
                     value=node.value if node.value else ast.Constant(value=None),
+                    lineno=0,
                 ),
                 self.get_event_call(function_exit_event),
             ],
@@ -517,25 +515,15 @@ class FunctionErrorEventFactory(FunctionEventFactory):
     def get_function(self):
         return "add_function_error_event"
 
-    def get_event_call(self, event: FunctionEnterEvent | FunctionErrorEvent):
-        return get_call(
-            self.get_function(),
-            event.file,
-            event.line,
-            event.id_,
-            event.function_id,
-            event.function,
-        )
-
     def visit_function(
         self, node: typing.Union[FunctionDef, AsyncFunctionDef]
     ) -> Injection:
         function_error_event = FunctionErrorEvent(
             self.file,
             node.lineno,
-            self.get_function_event_id(node, self.id_generator),
-            self.get_function_id(node),
+            self.event_id_generator.get_next_id(),
             node.name,
+            self.get_function_id(node),
         )
         return Injection(
             error=[self.get_event_call(function_error_event)],
@@ -553,13 +541,6 @@ class LoopEventFactory(PythonEventFactory):
     loops: typing.Dict[AST, int] = dict()
     loop_id: int = 0
 
-    def get_event_call(
-        self, event: typing.Union[LoopBeginEvent, LoopHitEvent, LoopEndEvent]
-    ):
-        return get_call(
-            self.get_function(), event.file, event.line, event.id_, event.loop_id
-        )
-
     @staticmethod
     def get_loop_id(node: AST) -> int:
         if node not in LoopEventFactory.loops:
@@ -588,7 +569,7 @@ class LoopBeginEventFactory(LoopEventFactory):
         loop_begin_event = LoopBeginEvent(
             self.file,
             node.lineno,
-            self.id_generator.get_next_id(),
+            self.event_id_generator.get_next_id(),
             self.get_loop_id(node),
         )
         return Injection(
@@ -604,7 +585,7 @@ class LoopHitEventFactory(LoopEventFactory):
         loop_hit_event = LoopHitEvent(
             self.file,
             node.lineno,
-            self.id_generator.get_next_id(),
+            self.event_id_generator.get_next_id(),
             self.get_loop_id(node),
         )
         return Injection(
@@ -620,7 +601,7 @@ class LoopEndEventFactory(LoopEventFactory):
         loop_end_event = LoopEndEvent(
             self.file,
             node.lineno,
-            self.id_generator.get_next_id(),
+            self.event_id_generator.get_next_id(),
             self.get_loop_id(node),
         )
         return Injection(
@@ -659,9 +640,7 @@ class UseEventFactory(PythonEventFactory):
         )
 
     def _get_std_call(self, event: UseEvent):
-        call = get_call(
-            self.get_function(), event.file, event.line, event.id_, event.var
-        )
+        call = get_call(self.get_function(), event.event_id)
         assert isinstance(call.value, Call)
         call.value.args.append(
             Call(
@@ -733,7 +712,9 @@ class UseEventFactory(PythonEventFactory):
         use_events = list()
         for use in uses:
             use_events.append(
-                UseEvent(self.file, node.lineno, self.id_generator.get_next_id(), use)
+                UseEvent(
+                    self.file, node.lineno, self.event_id_generator.get_next_id(), use
+                )
             )
         return Injection(
             pre=[self.get_event_call(e) for e in use_events], events=use_events
@@ -799,10 +780,16 @@ class UseEventFactory(PythonEventFactory):
         return self.visit_use(node.value)
 
     def visit_Global(self, node: Global) -> Injection:
-        return self.visit_use(node)
+        injection = self.visit_use(node)
+        injection.post = injection.pre
+        injection.pre = list()
+        return injection
 
     def visit_Nonlocal(self, node: Nonlocal) -> Injection:
-        return self.visit_use(node)
+        injection = self.visit_use(node)
+        injection.post = injection.pre
+        injection.pre = list()
+        return injection
 
 
 class ConditionEventFactory(PythonEventFactory):
@@ -812,10 +799,7 @@ class ConditionEventFactory(PythonEventFactory):
     def get_event_call(self, event: ConditionEvent):
         call = get_call(
             self.get_function(),
-            event.file,
-            event.line,
-            event.id_,
-            event.condition,
+            event.event_id,
         )
         assert isinstance(call.value, Call)
         call.value.args.append(
@@ -831,7 +815,69 @@ class ConditionEventFactory(PythonEventFactory):
         return Injection(pre=[var_assign], assign=var_use, events=events)
 
     def visit_While(self, node: While) -> Injection:
-        return self.visit_condition(node)
+        injection = self.visit_condition(node)
+        injection.body_last = injection.pre
+        return injection
 
     def visit_If(self, node: If) -> Injection:
         return self.visit_condition(node)
+
+
+class LenEventFactory(DefEventFactory):
+    def get_function(self):
+        return "add_len_event"
+
+    def get_check_for_len(self, event: LenEvent):
+        call = get_call(self.get_function(), event.event_id)
+        assert isinstance(call.value, Call)
+        call.value.args.append(
+            Call(
+                func=Attribute(
+                    value=Name(
+                        id=python_lib,  # enter lib
+                    ),
+                    attr="get_id",  # enter lib function
+                ),
+                args=[Name(id=event.var)],
+                keywords=[],
+            )
+        )
+        call.value.args.append(
+            Call(
+                func=Name(id="len"),
+                args=[Name(id=event.var)],
+                keywords=[],
+            )
+        )
+        return Try(
+            body=[call],
+            handlers=[
+                ExceptHandler(
+                    type=Tuple(
+                        elts=[
+                            Name(
+                                id="AttributeError",
+                            ),
+                            Name(
+                                id="TypeError",
+                            ),
+                            Name(
+                                id="NameError",
+                            ),
+                        ],
+                    ),
+                    name=None,
+                    body=[Pass()],
+                )
+            ],
+            orelse=[],
+            finalbody=[],
+        )
+
+    def get_event_call(self, event: LenEvent):
+        return self.get_check_for_len(event)
+
+    def get_event(self, node: typing.Union[Assign, AnnAssign, AugAssign], var: str):
+        return LenEvent(
+            self.file, node.lineno, self.event_id_generator.get_next_id(), var
+        )
