@@ -93,6 +93,8 @@ class ConfigTests(unittest.TestCase):
             config.visitor,
             config.passing,
             config.failing,
+            config.mapping,
+            config.instrument_include,
             config.instrument_exclude,
             config.instrument_working,
             config.runner,
