@@ -6,6 +6,7 @@ from sflkitlib.events import event
 
 from sflkit import instrument_config
 from sflkit.config import Config
+from sflkit.mapping import EventMapping
 from utils import BaseTest
 
 
@@ -21,7 +22,10 @@ class EventTests(BaseTest):
 
         subprocess.run([self.PYTHON, self.ACCESS], cwd=self.TEST_DIR)
 
-        events = event.load(os.path.join(self.TEST_DIR, self.TEST_PATH))
+        events = event.load(
+            os.path.join(self.TEST_DIR, self.TEST_PATH),
+            EventMapping.load(config).mapping,
+        )
         self.assertEqual(3, len(events))
         for i, e in enumerate(events):
             self.assertIsInstance(e, event.LineEvent, f"{e} is not a line event")
@@ -39,7 +43,10 @@ class EventTests(BaseTest):
 
         subprocess.run([self.PYTHON, self.ACCESS], cwd=self.TEST_DIR)
 
-        events = event.load(os.path.join(self.TEST_DIR, self.TEST_PATH))
+        events = event.load(
+            os.path.join(self.TEST_DIR, self.TEST_PATH),
+            EventMapping.load(config).mapping,
+        )
         self.assertEqual(5, len(events))
         lines = [1, 4, 5]
         line_i = 0
@@ -83,7 +90,10 @@ class EventTests(BaseTest):
         instrument_config(config)
 
         subprocess.run([BaseTest.PYTHON, BaseTest.ACCESS], cwd=BaseTest.TEST_DIR)
-        return event.load(os.path.join(BaseTest.TEST_DIR, BaseTest.TEST_PATH))
+        return event.load(
+            os.path.join(BaseTest.TEST_DIR, BaseTest.TEST_PATH),
+            EventMapping.load(config).mapping,
+        )
 
     def test_function(self):
         events = self._test_events("function_enter,function_exit")
@@ -330,6 +340,32 @@ class EventTests(BaseTest):
             self.assertEqual(ev, e.value, f"{e} has not correct value")
             self.assertEqual(exp, e.condition, f"{e} has not correct condition")
 
+    def test_len(self):
+        config = Config.create(
+            path=os.path.join(self.TEST_RESOURCES, self.TEST_LEN),
+            language="python",
+            events="len",
+            working=self.TEST_DIR,
+        )
+        instrument_config(config)
+
+        subprocess.run(
+            [self.PYTHON, self.ACCESS, "a", "b", "c"],
+            cwd=self.TEST_DIR,
+        )
+
+        events = event.load(
+            os.path.join(self.TEST_DIR, self.TEST_PATH),
+            EventMapping.load(config).mapping,
+        )
+        self.assertEqual(2, len(events))
+        for i, a, l, e in zip([4, 5], ["args", "a"], [3, 1], events):
+            self.assertIsInstance(e, event.LenEvent, f"{e} is not a line event")
+            self.assertEqual(self.ACCESS, e.file, f"{e} has not correct file")
+            self.assertEqual(i, e.line, f"{e} has not correct line")
+            self.assertEqual(a, e.var, f"{e} has not correct var")
+            self.assertEqual(l, e.length, f"{e} has not correct length")
+
 
 class SerializeEventsTest(BaseTest):
     @parameterized.expand(map(lambda x: (str(x), x), BaseTest.EVENTS))
@@ -338,24 +374,4 @@ class SerializeEventsTest(BaseTest):
 
     @parameterized.expand(map(lambda x: (str(x), x), BaseTest.EVENTS))
     def test_load(self, _, e):
-        args = e.dump()[1:]
-        if e.event_type == event.EventType.FUNCTION_EXIT:
-            if e.return_value == 1:
-                args[5] = b"\x80\x04K\x01."
-            elif e.return_value is None:
-                args[5] = "None"
-            elif e.return_value:
-                args[5] = "True"
-            elif not e.return_value:
-                args[5] = "False"
-        elif e.event_type == event.EventType.DEF:
-            if e.value == 1:
-                args[5] = b"\x80\x04K\x01."
-            elif e.value is None:
-                args[5] = "None"
-            elif e.value:
-                args[5] = "True"
-            elif not e.value:
-                args[5] = "False"
-
-        self.assertEqual(e, event.load_event(e.event_type, *args))
+        self.assertEqual(e, event.load_event(e.dump(), {e.event_id: e}))
