@@ -2,24 +2,14 @@ import atexit
 import os
 from pathlib import Path
 
-from parameterized import parameterized
-from sflkitlib.events import event, EventType
+from sflkitlib.events import EventType
 
 from sflkit import instrument_config, Config
 from utils import BaseTest
 
 
 class TestInstrumentation(BaseTest):
-    def test_complex_structure(self):
-        config = Config.create(
-            path=os.path.join(BaseTest.TEST_RESOURCES, "test_instrumentation"),
-            language="python",
-            events="line",
-            predicates="line",
-            working=BaseTest.TEST_DIR,
-            exclude="exclude_dir,exclude.py,excluded_file,"
-            + os.path.join("package", "exclude.py"),
-        )
+    def _test_complex_structure(self, config: Config):
         instrument_config(config)
         dst = Path(BaseTest.TEST_DIR)
         src = Path(BaseTest.TEST_RESOURCES, "test_instrumentation")
@@ -89,6 +79,102 @@ class TestInstrumentation(BaseTest):
                 s_content, d_content, f"{d} has the same content then {s}"
             )
 
+    def test_complex_structure_exclude(self):
+        self._test_complex_structure(
+            Config.create(
+                path=os.path.join(BaseTest.TEST_RESOURCES, "test_instrumentation"),
+                language="python",
+                events="line",
+                predicates="line",
+                working=BaseTest.TEST_DIR,
+                exclude=r"exclude_dir,exclude\.py,excluded_file,"
+                + os.path.join("package", r"exclude.py"),
+            )
+        )
+
+    def test_complex_structure_include(self):
+        self._test_complex_structure(
+            Config.create(
+                path=os.path.join(BaseTest.TEST_RESOURCES, "test_instrumentation"),
+                language="python",
+                events="line",
+                predicates="line",
+                working=BaseTest.TEST_DIR,
+                include=r"package,main\.py",
+                exclude=os.path.join("package", r"exclude\.py"),
+            )
+        )
+
+    def test_mapping_output(self):
+        instrument_config(
+            Config.create(
+                path=os.path.join(BaseTest.TEST_RESOURCES, "test_instrumentation"),
+                language="python",
+                events="line",
+                predicates="line",
+                working=BaseTest.TEST_DIR,
+                mapping_path="mapping.json",
+            )
+        )
+        mapping = Path("mapping.json")
+        self.assertTrue(mapping.exists())
+        self.assertTrue(mapping.is_file())
+
+    def test_instrument_exclude(self):
+        src = Path(BaseTest.TEST_RESOURCES, "test_exclude")
+        dst = Path(BaseTest.TEST_DIR)
+        instrument_config(
+            Config.create(
+                path=str(src),
+                language="python",
+                events="line",
+                predicates="line",
+                working=BaseTest.TEST_DIR,
+                include="included",
+                exclude=os.path.join("included", "excluded"),
+            )
+        )
+        included = dst / "included"
+        excluded = included / "excluded"
+        a = included / "a.py"
+        b = excluded / "b.py"
+        c = dst / "c.py"
+
+        included_src = src / "included"
+        excluded_src = included_src / "excluded"
+        a_src = included_src / "a.py"
+        b_src = excluded_src / "b.py"
+        c_src = src / "c.py"
+
+        self.assertTrue(included.exists())
+        self.assertTrue(included.is_dir())
+        self.assertTrue(excluded.exists())
+        self.assertTrue(excluded.is_dir())
+        self.assertTrue(a.exists())
+        self.assertTrue(a.is_file())
+        self.assertTrue(b.exists())
+        self.assertTrue(b.is_file())
+        self.assertTrue(c.exists())
+        self.assertTrue(c.is_file())
+
+        with open(a, "r") as fp:
+            a_content = fp.read()
+        with open(a_src, "r") as fp:
+            a_src_content = fp.read()
+        self.assertNotEqual(a_src_content, a_content)
+
+        with open(b, "r") as fp:
+            b_content = fp.read()
+        with open(b_src, "r") as fp:
+            b_src_content = fp.read()
+        self.assertEqual(b_src_content, b_content)
+
+        with open(c, "r") as fp:
+            c_content = fp.read()
+        with open(c_src, "r") as fp:
+            c_src_content = fp.read()
+        self.assertEqual(c_src_content, c_content)
+
 
 class TestLib(BaseTest):
     @classmethod
@@ -114,10 +200,6 @@ class TestLib(BaseTest):
     def setUp(self) -> None:
         self.events = None
 
-    def load(self) -> None:
-        self.lib.dump_events()
-        self.events = event.load(self.TEST_PATH)
-
     def tearDown(self) -> None:
         # noinspection PyBroadException
         try:
@@ -126,22 +208,6 @@ class TestLib(BaseTest):
             pass
         self.lib.reset()
 
-    @parameterized.expand(map(lambda x: (str(x), x), BaseTest.EVENTS))
-    def test_lib(self, _, e):
-        args = e.dump()[1:]
-        while "int" in args:
-            args[args.index("int")] = int
-        while "bool" in args:
-            args[args.index("bool")] = bool
-        while "none" in args:
-            args[args.index("none")] = type(None)
-        while "list" in args:
-            args[args.index("list")] = list
-        self.event_type_map[e.event_type](*args)
-        self.load()
-        self.assertEqual(1, len(self.events))
-        self.assertIn(e, self.events)
-
     def test_get_id(self):
         x = 10
         self.assertEqual(id(x), self.lib.get_id(x))
